import inquirer

from ts_cli.config.cli_config import CliConfig
from ts_cli.config.config import Config
from ts_cli.config.provider import Provider
from ts_cli.util.emit import emit_critical


class InteractiveConfig(Config):
    def __init__(self, args, *, interactive: bool):
        super().__init__()
        self._cli_config = CliConfig(args)
        self._interactive = interactive
        self.type = None
        self.namespace = None
        self.slug = None
        self.version = None

    def _resolve(
        self, provider: Provider, keys: list, skip_confirmation: bool = False
    ) -> dict:
        values: dict = {key: provider.get(key) or None for key in keys}
        if not self._interactive:
            return values
        values["confirmed"] = False
        while values.get("confirmed", False) is not True:
            inquiry = self._get_inquiry(values)
            if not skip_confirmation:
                inquiry.append(
                    inquirer.Confirm(
                        "confirmed",
                        message=self._get_correct_message,
                        default=False,
                    )
                )
            values = inquirer.prompt(inquiry)
            if values is None:
                raise emit_critical("Configuration interrupted")
            if skip_confirmation:
                values["confirmed"] = True
        return self._parse(values)

    def _parse(self, values: dict) -> dict:
        return values

    def _get_correct_message(self, answers: dict) -> str:
        return "Correct?"

    def _get_inquiry(self, existing_values: dict) -> list:
        """
        Returns a list of inquirer questions, using existing values as defaults
        :param existing_values:
        :return:
        """
        raise NotImplemented()
